from typing import Optional

import discord
from cpl_core.database.context import DatabaseContextABC
from cpl_discord.events import OnMessageABC
from cpl_discord.service import DiscordBotServiceABC

from bot_core.helper.event_checks import EventChecks
from bot_core.helper.log_message_helper import LogMessageHelper
from bot_core.logging.message_logger import MessageLogger
from bot_core.service.client_utils_service import ClientUtilsService
from bot_data.abc.client_repository_abc import ClientRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.model.user import User
from modules.base.abc.base_helper_abc import BaseHelperABC
from modules.base.configuration.base_server_settings import BaseServerSettings


class BaseOnMessageEvent(OnMessageABC):
    def __init__(
        self,
        logger: MessageLogger,
        bhs: BaseHelperABC,
        client_utils: ClientUtilsService,
        db: DatabaseContextABC,
        bot: DiscordBotServiceABC,
        users: UserRepositoryABC,
        clients: ClientRepositoryABC,
        servers: ServerRepositoryABC,
    ):
        OnMessageABC.__init__(self)
        self._logger = logger
        self._base_helper = bhs
        self._client_utils = client_utils
        self._bot = bot
        self._db = db
        self._bot = bot
        self._users = users
        self._clients = clients
        self._servers = servers

    def _append_received_message_count(self, g_id: int):
        try:
            self._clients.append_received_message_count(self._bot.user.id, g_id, 1)
            self._db.save_changes()
        except Exception as e:
            self._logger.error(__name__, f"Cannot edit client {self._bot.user.id}@{g_id}", e)

    def _handle_message_for_xp(self, message: discord.Message):
        dc_user_id = message.author.id
        try:
            server = self._servers.get_server_by_discord_id(message.guild.id)
        except Exception as e:
            self._logger.error(__name__, f"Cannot get server {message.guild.id}", e)
            return

        user: Optional[User] = None
        try:
            user = self._users.find_user_by_discord_id_and_server_id(dc_user_id, server.server_id)
        except Exception as e:
            self._logger.error(__name__, f"Cannot get user {dc_user_id}", e)
            return

        if user is None:
            self._logger.error(__name__, f"User not found {dc_user_id}")
            return

        settings: BaseServerSettings = self._base_helper.get_config(message.guild.id)
        if self._client_utils.is_message_xp_count_by_hour_higher_that_max_message_count_per_hour(
            message.created_at, user, settings
        ):
            return

        old_xp = user.xp
        user.xp += settings.xp_per_message
        self._users.update_user(user)
        self._db.save_changes()

        self._logger.debug(__name__, f"User {user} sent message. xp: from {old_xp} to {user.xp}")

    @EventChecks.check_is_ready()
    async def on_message(self, message: discord.Message):
        self._logger.debug(__name__, f"Module {type(self)} started")
        if message is None or message.guild is None:
            return

        self._logger.info(__name__, f"Received message: {LogMessageHelper.get_log_string(message)}")
        self._append_received_message_count(message.guild.id)

        if not message.author.bot:
            self._handle_message_for_xp(message)
        self._logger.debug(__name__, f"Module {type(self)} stopped")
