from datetime import datetime
from typing import Optional

import discord
from cpl_core.configuration import ConfigurationABC
from cpl_core.database.context import DatabaseContextABC
from cpl_core.logging import LoggerABC
from cpl_discord.events import OnVoiceStateUpdateABC

from bot_core.helper.event_checks import EventChecks
from bot_data.abc.known_user_repository_abc import KnownUserRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.user_joined_server_repository_abc import UserJoinedServerRepositoryABC
from bot_data.abc.user_joined_voice_channel_repository_abc import (
    UserJoinedVoiceChannelRepositoryABC,
)
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.model.server import Server
from bot_data.model.user import User
from bot_data.model.user_joined_voice_channel import UserJoinedVoiceChannel
from modules.base.abc.base_helper_abc import BaseHelperABC
from modules.base.configuration.base_server_settings import BaseServerSettings


class BaseOnVoiceStateUpdateEvent(OnVoiceStateUpdateABC):
    def __init__(
        self,
        config: ConfigurationABC,
        logger: LoggerABC,
        base_helper: BaseHelperABC,
        servers: ServerRepositoryABC,
        known_users: KnownUserRepositoryABC,
        users: UserRepositoryABC,
        user_joins: UserJoinedServerRepositoryABC,
        user_joins_vc: UserJoinedVoiceChannelRepositoryABC,
        db: DatabaseContextABC,
    ):
        OnVoiceStateUpdateABC.__init__(self)
        self._config = config
        self._logger = logger
        self._base_helper = base_helper
        self._servers = servers
        self._known_users = known_users
        self._users = users
        self._user_joins = user_joins
        self._user_joins_vc = user_joins_vc
        self._db = db

        self._logger.info(__name__, f"Module {type(self)} loaded")

    def _update_voice_state(self, joined: bool, dc_user_id: int, dc_channel_id: int, server: Server):
        user: Optional[User] = None
        try:
            user = self._users.get_user_by_discord_id_and_server_id(dc_user_id, server.server_id)
        except Exception as e:
            self._logger.error(__name__, f"Cannot get user {dc_user_id}", e)
            return

        if user is None:
            self._logger.error(__name__, f"User not found {dc_user_id}")
            return

        try:
            if joined:
                join = UserJoinedVoiceChannel(user, dc_channel_id, datetime.now())
                self._user_joins_vc.add_user_joined_voice_channel(join)
                self._db.save_changes()
                return

            settings: BaseServerSettings = self._base_helper.get_config(server.discord_server_id)

            join = self._user_joins_vc.get_active_user_joined_voice_channel_by_user_id(user.user_id)
            join.leaved_on = datetime.now()

            # ontime as hours
            ontime = round((join.leaved_on - join.joined_on).total_seconds() / 3600, 2)
            old_xp = user.xp
            user.xp += round(ontime * settings.xp_per_ontime_hour)

            self._user_joins_vc.update_user_joined_voice_channel(join)
            self._users.update_user(user)
            self._db.save_changes()

            self._logger.debug(
                __name__,
                f"User {user} leaved_on {join.leaved_on}. Ontime: {ontime}h | xp: from {old_xp} to {user.xp}",
            )
        except Exception as e:
            self._logger.error(__name__, f"Ontime validation failed", e)

    @EventChecks.check_is_ready()
    async def on_voice_state_update(
        self,
        member: discord.Member,
        before: discord.VoiceState,
        after: discord.VoiceState,
    ):
        self._logger.debug(__name__, f"Module {type(self)} started")
        self._logger.trace(
            __name__,
            f"Detected on_voice_state_update {member.id} from {before} to {after}",
        )
        settings: BaseServerSettings = self._base_helper.get_config(member.guild.id)
        server = self._servers.get_server_by_discord_id(member.guild.id)

        try:
            # join
            if (
                before.channel is None
                and after.channel is not None
                and after.channel.id not in settings.afk_channel_ids
            ):
                self._logger.trace(__name__, f"User {member.id} joined {after.channel}")
                self._update_voice_state(True, member.id, after.channel.id, server)

            # leave
            elif (
                before.channel is not None
                and after.channel is None
                and before.channel.id not in settings.afk_channel_ids
            ):
                self._logger.trace(__name__, f"User {member.id} left {before.channel}")
                self._update_voice_state(False, member.id, before.channel.id, server)

            # channel to channel
            elif before.channel is not None and after.channel is not None:
                # joined
                if before.channel.id in settings.afk_channel_ids and after.channel.id not in settings.afk_channel_ids:
                    self._logger.trace(__name__, f"User {member.id} joined {after.channel}")
                    self._update_voice_state(True, member.id, after.channel.id, server)

                # left
                elif after.channel.id in settings.afk_channel_ids and before.channel.id not in settings.afk_channel_ids:
                    self._logger.trace(__name__, f"User {member.id} left {before.channel}")
                    self._update_voice_state(False, member.id, before.channel.id, server)

                else:
                    self._logger.trace(__name__, f"User {member.id} switched to {after.channel}")
                    self._update_voice_state(False, member.id, before.channel.id, server)
                    self._update_voice_state(True, member.id, after.channel.id, server)
        except Exception as e:
            self._logger.error(__name__, f"Cannot handle voice state for user {member.id}", e)
