from typing import List as TList

import discord
from cpl_core.database.context import DatabaseContextABC
from cpl_discord.command import DiscordCommandABC
from cpl_translation import TranslatePipe
from discord import app_commands
from discord.ext import commands
from discord.ext.commands import Context

from bot_core.abc.client_utils_abc import ClientUtilsABC
from bot_core.abc.message_service_abc import MessageServiceABC
from bot_core.helper.command_checks import CommandChecks
from bot_core.logging.command_logger import CommandLogger
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.statistic_repository_abc import StatisticRepositoryABC
from modules.permission.abc.permission_service_abc import PermissionServiceABC
from modules.stats.service.statistic_service import StatisticService
from modules.stats.ui.add_statistic_form import AddStatisticForm


class StatsGroup(DiscordCommandABC):
    def __init__(
        self,
        logger: CommandLogger,
        message_service: MessageServiceABC,
        client_utils: ClientUtilsABC,
        translate: TranslatePipe,
        permission_service: PermissionServiceABC,
        statistic: StatisticService,
        servers: ServerRepositoryABC,
        stats: StatisticRepositoryABC,
        db: DatabaseContextABC,
    ):
        DiscordCommandABC.__init__(self)

        self._logger = logger
        self._client_utils = client_utils
        self._message_service = message_service
        self._t = translate
        self._permissions = permission_service
        self._statistic = statistic
        self._servers = servers
        self._stats = stats
        self._db = db

    @commands.hybrid_group()
    @commands.guild_only()
    async def stats(self, ctx: Context):
        pass

    @stats.command()
    @commands.guild_only()
    @CommandChecks.check_is_ready()
    @CommandChecks.check_is_member_moderator()
    async def list(self, ctx: Context, wait: int = None):
        self._logger.debug(__name__, f"Received command stats list {ctx}")

        if ctx.guild is None:
            return

        embed = discord.Embed(
            title=self._t.transform("modules.auto_role.list.title"),
            description=self._t.transform("modules.auto_role.list.description"),
            color=int("ef9d0d", 16),
        )

        server = self._servers.get_server_by_discord_id(ctx.guild.id)
        stats = self._stats.get_statistics_by_server_id(server.server_id)

        if stats.count() == 0:
            await self._message_service.send_ctx_msg(ctx, self._t.transform("modules.stats.list.nothing_found"))
            return

        statistics = ""
        descriptions = ""
        for statistic in stats:
            statistics += f"\n{statistic.name}"
            descriptions += f"\n{statistic.description}"

        embed.add_field(
            name=self._t.transform("modules.stats.list.statistic"),
            value=statistics,
            inline=True,
        )
        embed.add_field(
            name=self._t.transform("modules.stats.list.description"),
            value=descriptions,
            inline=True,
        )
        await self._message_service.send_ctx_msg(ctx, embed, wait_before_delete=wait)
        self._logger.trace(__name__, f"Finished command stats list")

    @stats.command()
    @commands.guild_only()
    @CommandChecks.check_is_ready()
    @CommandChecks.check_is_member_moderator()
    async def view(self, ctx: Context, name: str, wait: int = None):
        self._logger.debug(__name__, f"Received command stats view {ctx}:{name}")

        if ctx.guild is None:
            return

        try:
            server = self._servers.get_server_by_discord_id(ctx.guild.id)
            stats = self._stats.get_statistics_by_server_id(server.server_id)
            statistic = stats.where(lambda s: s.name == name).single()
            result = await self._statistic.execute(statistic.code, server)

            embed = discord.Embed(
                title=statistic.name,
                description=statistic.description,
                color=int("ef9d0d", 16),
            )

            for i in range(result.header.count()):
                header = result.header[i]
                value = ""
                for row in result.values:
                    value += f"\n{row[i]}"
                embed.add_field(name=header, value=value, inline=True)

            await self._message_service.send_ctx_msg(ctx, embed, wait_before_delete=wait)
        except Exception as e:
            self._logger.error(__name__, f"Cannot view statistic {name}", e)
            await self._message_service.send_ctx_msg(ctx, self._t.transform("modules.stats.view.failed"))

        self._logger.trace(__name__, f"Finished stats view command")

    @view.autocomplete("name")
    async def view_autocomplete(
        self, interaction: discord.Interaction, current: str
    ) -> TList[app_commands.Choice[str]]:
        server = self._servers.get_server_by_discord_id(interaction.guild.id)
        stats = self._stats.get_statistics_by_server_id(server.server_id)
        return [
            app_commands.Choice(name=f"{statistic.name}: {statistic.description}", value=statistic.name)
            for statistic in stats
        ]

    @stats.command()
    @commands.guild_only()
    @CommandChecks.check_is_ready()
    @CommandChecks.check_is_member_technician()
    async def add(self, ctx: Context, name: str):
        self._logger.debug(__name__, f"Received command stats add {ctx}: {name}")

        if ctx.guild is None:
            return

        server = self._servers.get_server_by_discord_id(ctx.guild.id)
        form = AddStatisticForm(
            server,
            self._stats,
            self._db,
            name,
            self._message_service,
            self._logger,
            self._t,
        )
        self._logger.trace(__name__, f"Finished stats add command")
        self._logger.trace(__name__, f"Started stats command form")
        await ctx.interaction.response.send_modal(form)

    @stats.command()
    @commands.guild_only()
    @CommandChecks.check_is_ready()
    @CommandChecks.check_is_member_technician()
    async def edit(self, ctx: Context, name: str):
        self._logger.debug(__name__, f"Received command stats edit {ctx}: {name}")

        try:
            server = self._servers.get_server_by_discord_id(ctx.guild.id)
            stats = self._stats.get_statistics_by_server_id(server.server_id)
            statistic = stats.where(lambda s: s.name == name).single()
            form = AddStatisticForm(
                server,
                self._stats,
                self._db,
                name,
                self._message_service,
                self._logger,
                self._t,
                code=statistic.code,
                description=statistic.description,
            )
            self._logger.trace(__name__, f"Finished stats edit command")
            self._logger.trace(__name__, f"Started stats command form")
            await ctx.interaction.response.send_modal(form)
        except Exception as e:
            self._logger.error(__name__, f"Cannot edit statistic {name}", e)
            await self._message_service.send_ctx_msg(ctx, self._t.transform("modules.stats.edit.failed"))

    @edit.autocomplete("name")
    async def edit_autocomplete(
        self, interaction: discord.Interaction, current: str
    ) -> TList[app_commands.Choice[str]]:
        server = self._servers.get_server_by_discord_id(interaction.guild.id)
        stats = self._stats.get_statistics_by_server_id(server.server_id)
        return [
            app_commands.Choice(name=f"{statistic.name}: {statistic.description}", value=statistic.name)
            for statistic in stats
        ]

    @stats.command()
    @commands.guild_only()
    @CommandChecks.check_is_ready()
    @CommandChecks.check_is_member_technician()
    async def remove(self, ctx: Context, name: str):
        self._logger.debug(__name__, f"Received command stats remove {ctx}: {name}")

        try:
            server = self._servers.get_server_by_discord_id(ctx.guild.id)
            statistic = self._stats.get_statistic_by_name(name, server.server_id)
            self._stats.delete_statistic(statistic)
            self._db.save_changes()
            await self._message_service.send_ctx_msg(ctx, self._t.transform("modules.stats.remove.success"))
            self._logger.trace(__name__, f"Finished stats remove command")
        except Exception as e:
            self._logger.error(__name__, f"Cannot remove statistic {name}", e)
            await self._message_service.send_ctx_msg(ctx, self._t.transform("modules.stats.remove.failed"))

    @remove.autocomplete("name")
    async def edit_autocomplete(
        self, interaction: discord.Interaction, current: str
    ) -> TList[app_commands.Choice[str]]:
        server = self._servers.get_server_by_discord_id(interaction.guild.id)
        stats = self._stats.get_statistics_by_server_id(server.server_id)
        return [
            app_commands.Choice(name=f"{statistic.name}: {statistic.description}", value=statistic.name)
            for statistic in stats
        ]
