import { Component, OnInit } from "@angular/core";
import { FormBuilder, FormControl, FormGroup } from "@angular/forms";
import { Router } from "@angular/router";
import { TranslateService } from "@ngx-translate/core";
import { LazyLoadEvent } from "primeng/api";
import { debounceTime, throwError } from "rxjs";
import { ConfirmationDialogService } from "src/app/services/confirmation-dialog/confirmation-dialog.service";
import { DataService } from "src/app/services/data/data.service";
import { SpinnerService } from "src/app/services/spinner/spinner.service";
import { ToastService } from "src/app/services/toast/toast.service";
import { Server } from "../../../../../models/data/server.model";
import { catchError } from "rxjs/operators";
import { Queries } from "../../../../../models/graphql/queries.model";
import { Page } from "../../../../../models/graphql/filter/page.model";
import { Sort } from "../../../../../models/graphql/filter/sort.model";
import { Query } from "../../../../../models/graphql/query.model";
import { SidebarService } from "../../../../../services/sidebar/sidebar.service";

@Component({
  selector: "app-dashboard",
  templateUrl: "./dashboard.component.html",
  styleUrls: ["./dashboard.component.scss"]
})
export class DashboardComponent implements OnInit {

  servers: Server[] = [];

  totalRecords!: number;

  filter = {
    name: ""
  };

  page: Page = {
    pageIndex: 0,
    pageSize: 10
  };

  sort!: Sort;

  filterForm!: FormGroup<{
    name: FormControl<string | null>,
  }>;

  constructor(
    private data: DataService,
    private spinnerService: SpinnerService,
    private toastService: ToastService,
    private confirmDialog: ConfirmationDialogService,
    private fb: FormBuilder,
    private translate: TranslateService,
    private router: Router,
    private sidebar: SidebarService,
  ) {
  }

  async ngOnInit(): Promise<void> {
    this.spinnerService.showSpinner();

    this.setFilterForm();
    this.loadNextPage();
  }

  setFilterForm() {
    this.filterForm = this.fb.group({
      name: [""]
    });

    this.filterForm.valueChanges.pipe(
      debounceTime(600)
    ).subscribe(async changes => {
      if (changes.name) {
        this.filter.name = changes.name;
      }

      if (this.page.pageSize)
        this.page.pageSize = 10;

      if (this.page.pageSize)
        this.page.pageIndex = 0;

      await this.loadNextPage();
    });
  }

  loadNextPage() {
    this.spinnerService.showSpinner();
    this.data.query<Query>(Queries.serversListQuery,{
      filter: this.filter,
      page: this.page,
      sort: this.sort,
    }).pipe(catchError(err => {
      this.spinnerService.hideSpinner();
      return throwError(() => err);
    })).subscribe(data => {
      this.totalRecords = data.serverCount;
      this.servers = data.servers;
      this.spinnerService.hideSpinner();
    });
  }

  nextPage(event: LazyLoadEvent) {
    this.page.pageSize = event.rows ?? 0;
    if (event.first != null && event.rows != null)
      this.page.pageIndex = event.first / event.rows;

    this.sort = {
      sortColumn: event.sortField ?? "",
      sortDirection: event.sortOrder === 1 ? "asc" : event.sortOrder === -1 ? "desc" : "asc"
    };

    if (event.filters) {
      // + "" => convert to string
      this.filter.name = event.filters["name"] ? event.filters["name"] + "" : "";
    }

    this.loadNextPage();
  }

  resetFilters() {
    this.filterForm.reset();
  }

  selectServer(server: Server) {
    this.sidebar.serverName$.next(server.name ?? "");
    this.router.navigate(["/server", server.id]);
  }

}
